/********************************************************************************
  uniprime7_mrna_extract_5prace_product.c
  ---------------------------------------
  Roderic Guigo Serra, 2003
  modified by Sarah Djebali and Sylvain Foissac on Dec 2006
  
  Input:
  ------    
  - a file with mrna sequences (mrnaseqfile.tbl) in tbl format and uppercase,
  - a file with the reverse complement of 5' race primer sequences (primerfile.tbl) in tbl format and uppercase,
  - an integer (maxnomrna) corresponding to the maximum number of mrna given in,
  - an integer (maxmrnaseqlg) corresponding to the maximum length of the mrna sequences given in, 
  - an integer (maxnoprimers) corresponding to the maximum number of primers given in,
  - an integer (termlen) corresponding to the number of nucleotides at the end of the alignment that will start the alignment. We want to start an alignment if there is at least 1 match in the last termlen nucleotides (eg: 4),
  - an integer (pleng) corresponding to the minimum percentage of the length of the alignment where we want a minimum percentage identity to report the alignment (eg: 60),
  - an integer (pidty) corresponding to the minimum percentage identity we want over pleng in order to report the alignment (eg: 95),
  - an integer (raceprod), either 0 or 1, 0 meaning that the user does not want to get the 5' race products corresponding to the 5' primer matches (speeds up computation).

  Output:
  ------
  Reports all the alignments of all the reverse complemented primer sequences against all the mrna sequence
  plus the 5' race products from such alignment in the mrna sequence if the users asked for them (raceprod!=0).
  
  Remark: can be used to report the non artefactual mrna 5' race products as well to report the artefactual 
  mrna 5' race products obtained by 3' race primers.
  Note: it is the FIRST termlenb nucleotides of the match (revcomp of primer against mrna subsequence)
  that count in order to start an alignment.

  Use:
  ----
  Used for all the transcriptome against a list of reverse complement of primers
  (later the program could do the reverse complement itself, but then add _revcomp
   to the primer name)

  Be careful: 
  ---------
  - all dna and mrna sequences must be upper-case
  - HERE the extracted sequence could be much longer than the primer itself
    that is why subseq is allocated  gxseqlength characters (we could make the
    number of characters allocated a function of the racetype entered by the user)
  - termlen specified in the command line must be smaller than the smallest primer

  Also see:
  ---------
  http://genome.imim.es/~sdjebali/research/Primer_specificity.html
  for some explanations
********************************************************************************/



#include <stdio.h>

#include <time.h>

#define MAXSTRINGG 200
#define PRIMERLENGTH 50
#define MAXSTRINGP 200


/* ./uniprime7_mrna_extract_5prace_product mrnaseqfile.tbl primerfile_revcomp.tbl maxnomrna maxmrnaseqlg maxnoprimers termlen pleng pidty racetype raceprod */
/* ./uniprime7_mrna_extract_5prace_product mrnaseqfile.tbl primerfile_revcomp.tbl 100000 10000000 2000 4 60 95 1 */
/* ./uniprime7_mrna_extract_5prace_product Tmp/Hs.seq.uniq_unigene.tbl Tmp/Q1_5prime_primers_revcomp.tbl 100000 10000000 2000 4 60 95 1 2> Err/Q1_5prime_unigene.err > Out/Q1_5prime_unigene.out &*/


void shortime(void)
{
  time_t mytime = time(0); /* not 'long' */
  fprintf(stderr,"time: %s ", ctime(&mytime));
  fflush(stderr);
}

main(int argc, char *argv[])
{
  FILE *file;
  char * * GxSeq;
  char * * nameGxSeq; 
  int * LengthGxSeq;
  char * * Primer;
  char * * namePrimer;
  int * LengthPrimer;
  
  char * subseq;
  long i,j,k,l,m,n,p,q, gxseqlength, maxnogseq;
  int maxnoprimer;
  int termlenb, pleng, pidty;

  int match, found, termlen;
  int raceprod, lgtoextract;

  /* usage */
  if (argc != 10)
    {
      fprintf(stderr,"UNIPRIME MRNA EXTRACT RACE PRODUCT: look for primers in a set of mrna sequences and extract the corresponding 5' race products\n");
      fprintf(stderr,"WARNING: primers and mrna sequences must be UPPER-CASE\n");
      fprintf(stderr,"Usage:\n");
      fprintf(stderr,"./uniprime7_mrna_extract_5prace_product mrnaseqfile.tbl primerfile.tbl maxnomrna maxmrnaseqlg maxnoprimers termlen pleng pidty raceprod\nraceprod is either 0 or 1, 0 meaning that the user does not want to get the 5' race products corresponding to the 5' primer matches (speeds up computation)\n");
      exit(1);
    }


   /***************************************************************************************************************
   read the mrna sequences and the command line arguments relative to them and compute the lengths of the sequences
   ***************************************************************************************************************/
  maxnogseq = atol (argv[3]);
  gxseqlength = atol (argv[4]);
  fprintf(stderr,"The maxnogseq given in argument is %u\n", maxnogseq);
  fprintf(stderr,"The gxseqlength given in argument is %u\n", gxseqlength);

  if ((file=fopen(argv[1], "r"))==NULL)
    {
      fprintf(stderr,"The mrna sequence file %s cannot be opened\n",argv[1]);
      exit(1);
    }


  /************************************************
   allocate GxSeq, nameGxSeq, LengthGxSeq and subseq
   *************************************************/
  if(!(GxSeq=(char * *) malloc(maxnogseq*sizeof(char *))))
    {
      fprintf(stderr,"uniprime: not enough memory\n");
      exit(1); 
    }

  if(!(nameGxSeq=(char * *) malloc(maxnogseq*sizeof(char *))))
    {
      fprintf(stderr,"uniprime: not enough memory\n");
      exit(1); 
    }

  if(!(LengthGxSeq=(int *) malloc(maxnogseq*sizeof(int))))
    {
      fprintf(stderr,"uniprime: not enough memory\n");
      exit(1); 
    }

  if(!(subseq=(char *) malloc(gxseqlength*sizeof(char))))
    {
      fprintf(stderr,"uniprime: not enough memory\n");
      exit(1); 
    }
  
  for(p=0; p<maxnogseq; p++)
    {
      if(!(GxSeq[p]= (char *) malloc(gxseqlength*sizeof(char))))
	{
	  fprintf(stderr,"uniprime: not enough memory\n");
	  exit(1);
	}
      if(!(nameGxSeq[p] =  (char *) malloc(MAXSTRINGG*sizeof(char))))
	{
	  fprintf(stderr,"uniprime: not enough memory\n");
	  exit(1);
	}
    }
  fprintf(stderr,"I have done the double allocation of GxSeq and nameGxSeq, and the simple allocation of LengthGxSeq and subseq\n");

    
  /****************************
  scan the mrna sequence file
  ****************************/
  q=0;
  while (fscanf(file, "%s %s\n", nameGxSeq[q], GxSeq[q]) !=EOF)
    {
      //fprintf(stderr,"I am in the mrna sequence file scan, q is %d and nameGxSeq[q] is %s\n", q, nameGxSeq[q]);
      q++;
    }
  fprintf(stderr,"I have done the mrna sequence file scan and q is %d\n", q); 

  for (p=0;p<q;p++)
    {
      LengthGxSeq[p] = strlen(GxSeq[p]); 
    }
  fprintf(stderr,"I have computed the length of each mrna sequence\n"); 


  
  /***************************************************************************************************************
   read the primer sequences and the command line arguments relative to them and compute the lengths of the sequences
   ***************************************************************************************************************/
  maxnoprimer = atoi (argv[5]);

  if ((file=fopen(argv[2], "r"))==NULL)
    {
      fprintf(stderr,"The primer file %s cannot be opened\n",argv[2]);
      exit(1);
    }


  /**********************************************
   allocate Primer, namePrimer and LengthPrimer
   **********************************************/
  if(!(Primer = (char * *)  malloc(maxnoprimer * sizeof(char *)))) 
    {
      fprintf(stderr,"uniprime: not enough memory\n");
      exit(1);
    }

  if(!(namePrimer = (char * *)  malloc(maxnoprimer * sizeof(char *)))) 
    {
      fprintf(stderr,"uniprime: not enough memory\n");
      exit(1);
    }

  if(!(LengthPrimer = (int *)  malloc(maxnoprimer * sizeof(int)))) 
    {
      fprintf(stderr,"uniprime: not enough memory\n");
      exit(1);
    }

  for(m=0; m<maxnoprimer; m++)
    {
      if(!(Primer[m]= (char *) malloc(PRIMERLENGTH * sizeof(char))))
	{
	  fprintf(stderr,"uniprime: not enough memory\n");
	  exit(1);
	}
      if(!(namePrimer[m] =  (char *) malloc(MAXSTRINGP * sizeof(char))))
	{
	  fprintf(stderr,"uniprime: not enough memory\n");
	  exit(1);
	}
    }
  fprintf(stderr,"I have done the double allocation of Primer and namePrimer, and the simple allocation of LengthPrimer\n");  


  /***************************************************************************************
   read the parameters for match finding: termlen, pleng and pidty from the command line
  *****************************************************************************************/
  termlenb = atoi (argv[6]); 
  pleng = atoi (argv[7]);
  pidty = atoi (argv[8]);
  

  /***************************************************
   read the raceprod parameter from the command line
  *****************************************************/
  raceprod = atoi (argv[9]);


  /*****************************
  scan the primer sequence file
  ******************************/
  n=0;
  while (fscanf(file, "%s %s\n", namePrimer[n], Primer[n]) !=EOF)
    {
      fprintf(stderr,"I am in the primer file scan, n is %d, namePrimer[n] is %s and Primer[n] is %s\n", n, namePrimer[n], Primer[n] );
      n++;
    }
  fprintf(stderr,"I have done the primer file scan and n is %d\n", n); 

  for (m=0;m<n;m++)
    {
      LengthPrimer[m] = strlen(Primer[m]);
    }
  fprintf(stderr,"I have computed the length of each primer\n"); 


  shortime();  //  system("date +%D\" \"%T\" \"%s >&2");



  /**********************************************************************************************
  start looking for matches between the primer sequences and subsequences of the mrna sequences. 
  More precisely for each primer examine the set of mrna sequences.
  It is the FIRST termlenb nucleotides of the matches that count in the case of a 5' race (here).
  Then when a match is found and raceprod is 1 then the subsequence of the mrna begining at the
  match is also reported.
  **********************************************************************************************/
  m=0;   // primer number 
  while(m<n)
    {
      fprintf(stderr,"Aligning primer # %d to mrna sequences ... ", m+1);
      p=0;   // mrna sequence number 
      while(p<q)
	{
	  /* fprintf(stderr,"%d.. ",p+1); */

	  i=0;        // position in current mrna sequence
	  while(i<=LengthGxSeq[p]-LengthPrimer[m])
	    {
	      termlen=-1;
	      for (j=i;j<i+termlenb;j++)  // for each genomic position we start to look at the termlenb FIRST nt of the primer
		{
		  /* from the first position of the primer if aligned at i to TERMLEN bases after */
		  /* check if there is an alignment that satisfies the condition */
		  termlen++; // from 0 to TERMLEN
		  match=0;
		  found=0;
		  k=j; // iterator along the seq to count matches, from 5' to 3'
		  l=termlen; // iterator along the primer, from 5' to 3'
		  if (GxSeq[p][k]==Primer[m][l]) // match with one of the TERMLEN last positions
		    { 
		      match=1;
		      while ((found==0)&&(k<=i+LengthPrimer[m]-1)) // try to extend the alignment until the last position of the primer
			{
			  k++;
			  l++;
			  match += (GxSeq[p][k]==Primer[m][l]);
			  if (((k-j+1)*100>=(LengthPrimer[m]*pleng))&&(match*100>pidty*(k-j+1))) 
			    {
			      switch (raceprod)
				{
				case 0:  /* no race product extraction done, we only report the match of the primer to the mrna sequence */
				  fprintf(stdout, "%s\t%s\t%s\t%u\t%u\t%u\t%u\t%d\t%d\n", namePrimer[m], Primer[m], nameGxSeq[p], i+1, i+LengthPrimer[m], j+1 , k+1 , termlen+1 , match); 
				  break;

				default: /* 5' race product -> extraction of the mrna from its begining to the end of the match with the primer */
				  lgtoextract=i+LengthPrimer[m];
				  strncpy(subseq,GxSeq[p],lgtoextract);
				  subseq[lgtoextract]='\0';
				  fprintf(stdout, "%s\t%s\t%s\t%u\t%u\t%u\t%u\t%d\t%d\t%s\n", namePrimer[m], Primer[m], nameGxSeq[p], i+1, i+LengthPrimer[m], j+1 , k+1 , termlen+1 , match, subseq); 	
				  break;
				}
			      found=1;
			    }
			}
		    }
		  if (found)  /* to prevent a match to be reported twice while it corresponds to the same mrna position (maximum TERMLEN times) */
		    {
		      break;
		    }
		}
	      i++;   // go from the 5' to the 3' of mrna sequence number p
	    }
	  p++;  // loop over all mrna sequences
	}
      fprintf(stderr,"done;\n");
      shortime();
      /*      system("date +%D\" \"%T\" \"%s >&2"); */
      m++;  // loop over all primer sequences
    }
  fprintf(stderr,"I am finished\n");
}
