#uspp_filter_3pracepr.sh
#simulates the racearray process with 3' race primers in order to find unspecific positive probes (USPP)
#and then use them to filter out input RACEfrags that would have resulted from mis-priming and/or cross-hybridisation.

#Note:
#Needs the gcc compiler
#The length of the largest mrna passed to the program must not exceed 10Mb

#Takes as input eleven input arguments:
#first: name of the mrna file (without path) in tbl format and uppercase letters, to be located in $IN
#second: name of the primer file (without path) in tbl format and uppercase letters, to be located in $IN
#third: name of the array probe file in tbl format (without path) and upper case letters, to be located in $IN
#fourth: any integer different from 0 if you want to launch the 3' race production
#fifth: any integer different from 0 if you want to launch the comparaison of the 3' race products to the highlighted probes
#sixth: any integer different from 0 if it is the first time you are launching this script with the primers of the second argument
#seventh: name of the file (without path) containing the correspondance between primer, gene and pool (prname [tab] genename [tab] poolnumber), to be located in $IN
#eighth: name of the file (without path) containing the annotated exons in bed format, to be located in $IN
#nineth: integer corresponding to the total number of pools for the primers 
#tenth: name of the file containing the RACEfrags (without path) to pass to the uspp filter, in gff format with pool number in position given in 11th argument, to be located in $IN
#eleventh: integer corresponding to the position (field) of the pool number of RACEFrags in the RACEfrag gff file given as the 10th argument

#Outputs several files, the main one being in the directory where the script is launched and being called as the RF file given in the 10th argument followed by the suffix _overuspp3p_sorted.gff


#Example:
#./uspp_filter_3pracepr.sh human.rna.tbl Q2_primers.tbl ENCODE_2R_v01-2_NCBIv35.tbl 1 1 1 q2pr_gene_pool.txt 44regions_CHR_coord_exons_sorted.bed 10 RFassignment99.7_better_overbadprobes_withclass.gff 18 > out1 2> err1 &
#39116 mrnas, 331 primers, 2178956 probes, 10 pools, 17740 RACEfrags


#Directory for source programs
SRC=$PWD/src
#Directory for executables 
BIN=$PWD/bin
#Directory for the data
IN=$PWD/in
#Directory for error outputs
ERR=$PWD/err
#Directory for standard outputs
OUT=$PWD/out
#Directory for the awk scripts
AWK=$PWD/awk


#Recompile the C programs in order for the executables to be compatible with the machine on which they are lauched
echo "I am recompiling the C programs in order for the executables to be compatible with the present machine"
gcc -o $BIN/uniprime7_mrna_extract_3prace_product $SRC/uniprime7_mrna_extract_3prace_product.c
gcc -o $BIN/uniprime7_mrna_get_highlighted_probes_flex $SRC/uniprime7_mrna_get_highlighted_probes_flex.c
echo "done"


#########################  generate the 3' race products obtained by the 3' race primers           #################################
#########################  and compare both strands of these products to the tiling array probes   #################################
#For launching the production of the 3' race products
if [ -n $4 ] && [ $4 != 0 ]; then
echo "I am writing the comparison scripts to retrieve the 3' race products from the primers and the mrnas"
#Be careful we may have to augment the memory allocation
#usage: mrnaseqfile.tbl primerfile.tbl maxnomrna maxmrnaseqlg maxnoprimers termlen pleng pidty raceprod
#the size of the longest mrna sequence is supposed to be 10Mb  
maxnomrna=`wc -l $IN/$1 | awk '{print $1}'`
maxnoprimers=`wc -l $IN/$2 | awk '{print $1}'`
$BIN/uniprime7_mrna_extract_3prace_product $IN/$1 $IN/$2 $((maxnomrna+1)) 10000000 $((maxnoprimers+1)) 4 60 95 1 2> $ERR/$2_$1.err > $OUT/$2_$1.out
echo "done"
fi

#For computing some numbers about the race products
echo "Total number of 3' race products obtained:"
wc -l $OUT/$2_$1.out | awk '{print $1}'
echo "Here are the (3' race) primers represented by these products:"
awk '{print $1}' $OUT/$2_$1.out | sort -n | uniq -c      
echo "And they are a total of:"  
awk '{print $1}' $OUT/$2_$1.out | sort -n | uniq -c | wc -l 



#########################  compare the 3' race products (two strands) to each probe ################################
#For formatting and reverse complementing the 3' race products before comparing them to the probes 
echo "I am preparing the two strands of the 3' race products to be compared to the probes"
awk '{print $1"_"$2"_"$3"_"$4"_"$5"_"$6"_"$7"_"$8"_"$9"\t"$10}' $OUT/$2_$1.out > $OUT/$2_$1.tbl
$BIN/tblrevcomp $OUT/$2_$1.tbl > $OUT/$2_$1_revcomp.tbl 
echo "done"

#For launching comparison of the two strands of the 3' race products to the probes
if [ -n $5 ] && [ $5 != 0 ]; then
echo "I am launching the comparison scripts to retrieve the highlighted probes from the forward strands of the 3' race products"
#Be careful here we may need to augment the memory allocation
#usage: raceprodseqfile.tbl probefile.tbl maxnoraceprod maxraceprodseqlg maxnoprobes maxnbmismatch
#the size of the longest mrna race product sequence is supposed to be 10Mb
maxnoraceprodfw=`wc -l $OUT/$2_$1.tbl | awk '{print $1}'`
maxnoraceprodrv=`wc -l $OUT/$2_$1_revcomp.tbl | awk '{print $1}'`
maxnoprobes=`wc -l $IN/$3 | awk '{print $1}'`
$BIN/uniprime7_mrna_get_highlighted_probes_flex $OUT/$2_$1.tbl $IN/$3 $((maxnoraceprodfw+1)) 10000000 $((maxnoprobes+1)) 1 > $OUT/out_$3_$2_$1.out 2> $ERR/err_$3_$2_$1.err
echo "done"
echo "I am launching the comparison scripts to retrieve the highlighted probes from the reverse strands of the 3' race products"
$BIN/uniprime7_mrna_get_highlighted_probes_flex $OUT/$2_$1_revcomp.tbl $IN/$3 $((maxnoraceprodrv+1)) 10000000 $((maxnoprobes+1)) 1 > $OUT/out_$3_$2_$1_revcomp.out 2> $ERR/err_$3_$2_$1_revcomp.err
echo "done"
fi


#For analysing the probe comparison results
#Of 3' race product forward strands
if [ -f $OUT/out_$3_$2_$1.out ]; then 
echo "Number of probes highlighted by forward strand of 3' race products:"
wc -l $OUT/out_$3_$2_$1.out | awk '{print $1}'
echo "Here are the corresponding primers:"
awk '{split($3, a, "_"); print a[1]}' $OUT/out_$3_$2_$1.out | sort -n | uniq -c
echo "which are in total:"
awk '{split($3, a, "_"); print a[1]}' $OUT/out_$3_$2_$1.out | sort -n | uniq -c | wc -l | awk '{print $1}'
fi
#Of 3' race product reverse strands
if [ -f $OUT/out_$3_$2_$1_revcomp.out ]; then 
echo "Number of probes highlighted by reverse strands of 3' race products:"
wc -l $OUT/out_$3_$2_$1_revcomp.out | awk '{print $1}'
echo "Here are the corresponding primers:"
awk '{split($3, a, "_"); print a[1]}' $OUT/out_$3_$2_$1_revcomp.out | sort -n | uniq -c
echo "which are in total:"
awk '{split($3, a, "_"); print a[1]}' $OUT/out_$3_$2_$1_revcomp.out | sort -n | uniq -c | wc -l | awk '{print $1}'
fi


#For gathering results of the probe comparison done with 3' race product forward and reverse strands
#these are the positive probes
if [ -f $OUT/out_$3_$2_$1.out ] && [ -f $OUT/out_$3_$2_$1_revcomp.out ]; then 
cat $OUT/out_$3_$2_$1.out $OUT/out_$3_$2_$1_revcomp.out > $OUT/out_$3_$2_$1_all.out
rm $OUT/out_$3_$2_$1.out $OUT/out_$3_$2_$1_revcomp.out
echo "Number of probes highlighted by both strands of 3 race products:"
wc -l $OUT/out_$3_$2_$1_all.out | awk '{print $1}'
echo "Here are the corresponding primers:"
awk '{split($3, a, "_"); print a[1]}' $OUT/out_$3_$2_$1_all.out | sort -n | uniq -c
echo "which are in total:"
awk '{split($3, a, "_"); print a[1]}' $OUT/out_$3_$2_$1_all.out | sort -n | uniq -c | wc -l | awk '{print $1}'
fi

#For getting the positive probes in gff format with indication of gene name and pool number of originating primer
if [ -f $OUT/out_$3_$2_$1_all.out ]; then 
echo "I am getting the positive probes in gff format with indication of gene name and pool number of originating primer"
awk -v fileRef=$IN/$7 'BEGIN{while(getline < fileRef >0){gene[$1]=$2; pool[$1]=$3;}} {split($1,a,"_"); split($3,b,"_"); if(a[1]=="f") str="+"; else str="-"; print a[2]"\tRFsim\tprobe\t"a[3]"\t"(a[3]+24)"\t.\t"str"\t.\tname: "$1" seq: "$2" rprod: "$3" gnname: "gene[b[1]]" pool: "pool[b[1]];}' $OUT/out_$3_$2_$1_all.out | sort -k1,1 -k4,4n -k5,5n > $OUT/out_$3_$2_$1_all_sorted.gff
echo "done"
fi

#For knowing whether each positive probe is a uspp3p, ie not overlapping any exon of the gene of the originating primer
if [ -f $OUT/out_$3_$2_$1_all_sorted.gff ]; then 
echo "I am computing whether each positive probe is a uspp3p, ie not overlapping any exon of the gene of the originating primer"
sort -k1,1 -k2,2n -k3,3n $IN/$8 > $PWD/$8\_sorted_tmp
awk -v fldgn=16 -v toadd=exofprgn -v fileRef=$PWD/$8\_sorted_tmp -f $AWK/overlap_bed_precise.awk $OUT/out_$3_$2_$1_all_sorted.gff | awk '$NF==0' > $OUT/out_$3_$2_$1_all_sorted_uspp3p.gff
echo "done"
fi


#For knowing whether each RF overlaps a uspp3p of its pool
if [ -f $OUT/out_$3_$2_$1_all_sorted_uspp3p.gff ]; then
echo "I am computing whether each RF overlaps a uspp3p of its pool"
#Divide the uspp3p file into as many sorted files as there are pools ($9)
for i in $(seq 1 $9); do awk -v fldpool=18 -v p=$i '$fldpool==p' $OUT/out_$3_$2_$1_all_sorted_uspp3p.gff > $OUT/out_$3_$2_$1_all_sorted_uspp3p_$i.gff; done
#Do the actual comparison between the RF and the uspp3p, pool by pool and output the resulting gff file in the current directory
for i in $(seq 1 $9); do awk -v fldpool=${11} -v p=$i '$fldpool==p' $IN/${10} | sort -k1,1 -k4,4n -k5,5n | awk -v toadd=uspp3p -v fileRef=$OUT/out_$3_$2_$1_all_sorted_uspp3p_$i.gff -f $AWK/overlap_better_gff.awk; done | sort -k1,1 -k4,4n -k5,5n > $PWD/${10}\_overuspp3p_sorted.gff
#Removes intermediate files
for i in $(seq 1 $9); do rm $OUT/out_$3_$2_$1_all_sorted_uspp3p_$i.gff; done
echo "done"
fi


#Remove unecessary files
rm $PWD/$8\_sorted_tmp